﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StoragesContracts;
using LawFirmContracts.ViewModels;
using LawFirmDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;


namespace LawFirmDatabaseImplement.Implements
{
    public class DocumentStorage : IDocumentStorage
    {
        public List<DocumentViewModel> GetFullList()
        {
            using var context = new LawFirmDatabase();
            return context.Documents
             .Include(x => x.Blanks)
            .ThenInclude(x => x.Blank)
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<DocumentViewModel> GetFilteredList(DocumentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.DocumentName))
            {
                return new();
            }
            using var context = new LawFirmDatabase();
            return context.Documents
             .Include(x => x.Blanks)
            .ThenInclude(x => x.Blank)
            .Where(x => x.DocumentName.Contains(model.DocumentName))
            .ToList()
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public DocumentViewModel? GetElement(DocumentSearchModel model)
        {
            if (string.IsNullOrEmpty(model.DocumentName) &&
            !model.Id.HasValue)
            {
                return null;
            }
            using var context = new LawFirmDatabase();
            return context.Documents
             .Include(x => x.Blanks)
             .ThenInclude(x => x.Blank)
             .FirstOrDefault(x => (!string.IsNullOrEmpty(model.DocumentName) &&
            x.DocumentName == model.DocumentName) ||
            (model.Id.HasValue && x.Id ==
            model.Id))
            ?.GetViewModel;
        }
        public DocumentViewModel? Insert(DocumentBindingModel model)
        {
            using var context = new LawFirmDatabase();
            var newDocument = Document.Create(context, model);
            if (newDocument == null)
            {
                return null;
            }
            context.Documents.Add(newDocument);
            context.SaveChanges();
            return newDocument.GetViewModel;
        }
        public DocumentViewModel? Update(DocumentBindingModel model)
        {
            using var context = new LawFirmDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var product = context.Documents.FirstOrDefault(rec =>
                rec.Id == model.Id);
                if (product == null)
                {
                    return null;
                }
                product.Update(model);
                context.SaveChanges();
                product.UpdateBlanks(context, model);
                transaction.Commit();
                return product.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public DocumentViewModel? Delete(DocumentBindingModel model)
        {
            using var context = new LawFirmDatabase();
            var element = context.Documents
            .Include(x => x.Blanks)
            .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Documents.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }

}
