﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StoragesContracts;
using LawFirmContracts.ViewModels;
using LawFirmDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace LawFirmDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new LawFirmDatabase();

            var element = context.Orders
                .FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();

                return GetViewModel(element);
            }

            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new LawFirmDatabase();

            return GetViewModel(context.Orders
                .Include(x => x.Document)
                .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id)));
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }

            using var context = new LawFirmDatabase();

            return context.Orders
                .Include(x => x.Document)
                .Where(x => x.Id == model.Id)
                .Select(x => GetViewModel(x))
                .ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new LawFirmDatabase();

            return context.Orders
                .Include(x => x.Document)
                .Select(x => GetViewModel(x))
                .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }

            using var context = new LawFirmDatabase();

            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
						  .Include(x => x.Document)
						  .FirstOrDefault(x => x.Id == newOrder.Id)
						  ?.GetViewModel; 
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new LawFirmDatabase();

            var order = context.Orders
				.Include(x => x.Document)
				.FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }

            order.Update(model);
            context.SaveChanges();

            return GetViewModel(order);
        }

        private static OrderViewModel GetViewModel(Order order)
        {
            var viewModel = order.GetViewModel;
            using var context = new LawFirmDatabase();
            var element = context.Documents.FirstOrDefault(x => x.Id == order.DocumentId);
            viewModel.DocumentName = element.DocumentName;
            return viewModel;
        }
    }
}
