﻿using LawFirmContracts.BindingModels;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StoragesContracts;
using LawFirmContracts.ViewModels;
using LawFirmDatabaseImplement.Models;
using LawFirmDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LawFirmDatabaseImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName) &&
			!model.Id.HasValue)
			{
				return null;
			}
			using var context = new LawFirmDatabase();
			return context.Shops
			 .Include(x => x.Documents)
			 .ThenInclude(x => x.Document)
			 .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) &&
			x.ShopName == model.ShopName) ||
			(model.Id.HasValue && x.Id ==
			model.Id))
			?.GetViewModel;
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.ShopName))
			{
				return new();
			}
			using var context = new LawFirmDatabase();
			return context.Shops
			 .Include(x => x.Documents)
			.ThenInclude(x => x.Document)
			.Where(x => x.ShopName.Contains(model.ShopName))
			.ToList()
			.Select(x => x.GetViewModel)
			.ToList();
		}

		public List<ShopViewModel> GetFullList()
		{
			using var context = new LawFirmDatabase();
			return context.Shops.Include(x => x.Documents)
			.ThenInclude(x => x.Document)
			.ToList()
			.Select(x => x.GetViewModel)
			.ToList();
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			using var context = new LawFirmDatabase();
			var newProduct = Shop.Create(context, model);
			if (newProduct == null)
			{
				return null;
			}
			context.Shops.Add(newProduct);
			context.SaveChanges();
			return newProduct.GetViewModel;
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			using var context = new LawFirmDatabase();
			using var transaction = context.Database.BeginTransaction();
			try
			{
				var shop = context.Shops.FirstOrDefault(rec =>
				rec.Id == model.Id);
				if (shop == null)
				{
					return null;
				}
				shop.Update(model);
				context.SaveChanges();
				shop.UpdateDocuments(context, model);
				transaction.Commit();
				return shop.GetViewModel;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}
		public ShopViewModel? Delete(ShopBindingModel model)
		{
			using var context = new LawFirmDatabase();
			var element = context.Shops
			.Include(x => x.Documents)
			.FirstOrDefault(rec => rec.Id == model.Id);
			if (element != null)
			{
				context.Shops.Remove(element);
				context.SaveChanges();
				return element.GetViewModel;
			}
			return null;
		}
		public bool SellDocuments(IDocumentModel model, int count)
		{
			using var context = new LawFirmDatabase();
			using var transaction = context.Database.BeginTransaction();
			try
			{
				List<Shop> shopsWithDocument = context.Shops.Include(x => x.Documents).ThenInclude(x => x.Document).Where(x => x.Documents.Any(x => x.DocumentId == model.Id)).ToList();
				foreach (var shop in shopsWithDocument)
				{
					int carInShopCount = shop.ShopDocuments[model.Id].Item2;
					if (count - carInShopCount >= 0)
					{
						count -= carInShopCount;
						context.ShopDocuments.Remove(shop.Documents.FirstOrDefault(x => x.DocumentId == model.Id)!);
						shop.ShopDocuments.Remove(model.Id);
					}
					else
					{
						shop.ShopDocuments[model.Id] = (model, carInShopCount - count);
						count = 0;
						shop.UpdateDocuments(context, new()
						{
							Id = shop.Id,
							ShopDocuments = shop.ShopDocuments
						});
					}
					if (count == 0)
					{
						context.SaveChanges();
						transaction.Commit();
						return true;
					}
				}
				transaction.Rollback();
				return false;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}
	}
}
